"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_1 = require("lodash");
exports.isClickEvent = function (event) {
    if (!event || !event.data)
        return false;
    var data = event.data;
    var isMouseInteraction = data.source === 2;
    var isClick = data.type === 2;
    return isMouseInteraction && isClick && !!data.isTrusted;
};
exports.isScrollEvent = function (event) {
    if (!event || !event.data)
        return false;
    var data = event.data;
    var isScroll = data.source === 3;
    var isPageBody = data.id === 1;
    return isScroll && isPageBody;
};
exports.isInputEvent = function (event) {
    if (!event || !event.data)
        return false;
    var data = event.data;
    var isInput = data.source === 5;
    return isInput && !!data.isTrusted && !lodash_1.isNil(data.text);
};
exports.findActionEvents = function (events) {
    return events.filter(function (event) {
        return exports.isClickEvent(event) || exports.isInputEvent(event) || exports.isScrollEvent(event);
    });
};
exports.getEventAction = function (event) {
    if (exports.isClickEvent(event))
        return "click";
    if (exports.isInputEvent(event))
        return "input";
    if (exports.isScrollEvent(event))
        return "scroll";
    throw new Error("Action not found for event " + event);
};
exports.groupEventSequences = function (events) {
    var sequences = [];
    var sequence = null;
    events.forEach(function (event) {
        var action = exports.getEventAction(event);
        var eventXpath = event.data.xpath || null;
        if (!sequence ||
            action !== sequence.action ||
            eventXpath !== sequence.xpath) {
            sequence = {
                action: action,
                events: [event],
                xpath: eventXpath
            };
            sequences.push(sequence);
        }
        else {
            sequence.events.push(event);
        }
    });
    return sequences;
};
exports.buildClickSteps = function (clickSequence, nextSequence) {
    var steps = [];
    var willInputNext = nextSequence &&
        nextSequence.action === "input" &&
        nextSequence.xpath === clickSequence.xpath;
    clickSequence.events.forEach(function (click, i) {
        var isLastClick = i === clickSequence.events.length - 1;
        if (isLastClick && willInputNext)
            return;
        steps.push({
            action: "click",
            target: click.data.properties,
            pageId: click.pageId
        });
    });
    return steps;
};
exports.buildInputStep = function (inputSequence) {
    var lastInput = lodash_1.last(inputSequence.events);
    return {
        action: "input",
        target: lastInput.data.properties,
        pageId: lastInput.pageId,
        value: lastInput.data.text
    };
};
exports.buildScrollStep = function (scrollSequence) {
    var firstScroll = lodash_1.first(scrollSequence.events);
    var lastScroll = lodash_1.last(scrollSequence.events);
    if (firstScroll.data.y === lastScroll.data.y)
        return null;
    return {
        action: "scroll",
        target: { xpath: "scroll" },
        pageId: lastScroll.pageId,
        scrollDirection: firstScroll.data.y < lastScroll.data.y ? "down" : "up",
        scrollTo: lastScroll.data.y
    };
};
exports.buildSequenceSteps = function (eventSequences) {
    var steps = [];
    eventSequences.forEach(function (eventSequence, i) {
        if (eventSequence.action === "input") {
            steps.push(exports.buildInputStep(eventSequence));
        }
        else if (eventSequence.action === "scroll") {
            var scrollStep = exports.buildScrollStep(eventSequence);
            if (scrollStep)
                steps.push(scrollStep);
        }
        else if (eventSequence.action === "click") {
            var nextSequence = eventSequences[i + 1] || null;
            steps = steps.concat(exports.buildClickSteps(eventSequence, nextSequence));
        }
    });
    return steps;
};
exports.buildSteps = function (events) {
    var actionEvents = exports.findActionEvents(events);
    var eventSequences = exports.groupEventSequences(actionEvents);
    var steps = exports.buildSequenceSteps(eventSequences);
    return steps;
};
//# sourceMappingURL=buildSteps.js.map